from typing import Union
import us

from pypums.surveys import _clean_year

BASE_URL = "https://www2.census.gov/programs-surveys/" 

def build_acs_url(
    year: Union[int, str] = "2017",
    survey: Union[str, int] = "1-Year",
    person_or_household: str = "person",
    state: str = "California",
):
    """
    Builds CENSUS FTP-server URL where you can download ACS 1-, 3-, or 5- year estimates. 
    """
    _BASE_URL = BASE_URL + "acs/data/pums/"
    _unit = person_or_household[0].lower()
    _state_abbr = us.states.lookup(state).abbr.lower()

    if "5" in str(survey):
        _survey = "5-Year"
    elif "3" in str(survey):
        _survey = "3-Year"
    else:
        _survey = "1-Year"
    _year = _clean_year(year)

    def _ONE_THREE_OR_FIVE_YEAR(_survey: str = _survey, _year: int = _year) -> str:
        """
        Fixes URL part for survey. Some years don't have 3-Year surveys.
        If year <= 2006, _survey == ''.
        From 2007-2008, _survey can be either 1 or 3 years.
        From 2009-2013, _survey can be either 1, 3, or 5 years.
        From 2013 onward, only 1 or 5 years.
        """
        if _year <= 2006:
            if _survey != "1-Year":
                print(
                    "Prior to 2007, only 1-Year ACS are available, defaulting to 1-Year"
                )
                _survey = ""
        elif (2007 <= _year) and (_year <= 2008):
            if _survey == "5-Year":
                print(f"There is no 5-Year ACS for {_year}, defaulting to 3-Year")
                _survey = "3-Year"
        elif _year >= 2014:
            if _survey == "3-Year":
                print(f"There is no 3-Year ACS for {_year}, defaulting to 5-Year")
                _survey = "5-Year"
        return _survey

    _survey = _ONE_THREE_OR_FIVE_YEAR(_survey, _year)

    SURVEY_URL = (
        _BASE_URL
        + str(_year)
        + "/"
        + _survey
        + "/"
        + f"csv_{_unit}{_state_abbr}"
        + ".zip"
    )
    return SURVEY_URL



from dataclasses import dataclass, field
from typing import Union
import us

_BASE_URL = "https://www2.census.gov/programs-surveys/"

# @dataclass
# class Survey:
#     year: Union[int, str]
#     state: str
#     survey: Union[int, str]
#     person_or_household: str


def _clean_year(year: Union[int, str]) -> int:
    ## YEAR
    try:
        year = int(year)
    except ValueError:
        raise ValueError("year must be a number.")

    if (0 <= year) & (year <= 17):
        year += 2000

    if not ((2000 <= year) & (year <= 2017)):
        raise ValueError("Year must be between 2000 and 2017.")
    return year


@dataclass
class ACS:
    year: Union[int, str] = 2017
    state: str = "California"
    survey: Union[int, str] = "1-Year"
    person_or_household: str = "person"
    _BASE_URL: str = field(default=_BASE_URL + "acs/data/pums/", repr=False)

    def _SURVEY_URL_MAKER(self):
        """
        Builds url from which to retrieve data from Census FTP server.
        """
        _unit = self.person_or_household[0].lower()
        _state_abbr = us.states.lookup(self.state).abbr.lower()

        if "5" in str(self.survey):
            _survey = "5-Year"
        elif "3" in str(self.survey):
            _survey = "3-Year"
        else:
            _survey = "1-Year"
        _year = _clean_year(self.year)

        def _ONE_THREE_OR_FIVE_YEAR(_survey: str = _survey, _year: int = _year) -> str:
            """
            Fixes URL part for survey. Some years don't have 3-Year surveys.
            If year <= 2006, _survey == ''.
            From 2007-2008, _survey can be either 1 or 3 years.
            From 2009-2013, _survey can be either 1, 3, or 5 years.
            From 2013 onward, only 1 or 5 years.
            """
            if _year <= 2006:
                if _survey != "1-Year":
                    print(
                        "Prior to 2007, only 1-Year ACS are available, defaulting to 1-Year"
                    )
                    _survey = ""
            elif (2007 <= _year) and (_year <= 2008):
                if _survey == "5-Year":
                    print(f"There is no 5-Year ACS for {_year}, defaulting to 3-Year")
                    _survey = "3-Year"
            elif _year >= 2014:
                if _survey == "3-Year":
                    print(f"There is no 3-Year ACS for {_year}, defaulting to 5-Year")
                    _survey = "5-Year"
            return _survey

        _survey = _ONE_THREE_OR_FIVE_YEAR(_survey, _year)

        self._SURVEY_URL = (
            _BASE_URL
            + str(_year)
            + "/"
            + _survey
            + "/"
            + f"csv_{_unit}{_state_abbr}"
            + ".zip"
        )
        return None

    def __post_init__(self):
        self._SURVEY_URL_MAKER()



# -*- coding: utf-8 -*-

"""Console script for acs_download."""
import sys
import click

from pypums import pypums


@click.command(
    help="Download ACS PUMS entire data files from US Census Bureau's FTP server."
)
@click.option(
    "--year",
    required=True,
    prompt="2000-2017",
    default=2017,
    show_default=True,
    type=click.IntRange(min=2000, max=2017, clamp=True),
)
@click.option(
    "--state",
    required=True,
    prompt="State",
    default="Alaska",
    show_default=True,
    type=click.STRING,
)
@click.option(
    "--survey",
    prompt="1-, 3-, or 5-year",
    default="1-year",
    show_default=True,
    type=click.Choice(choices=("1-year","3-year", "5-year"), case_sensitive=False),
)
@click.option(
    "--person-or-household",
    prompt="person or household",
    default="person",
    show_default=True,
    type=click.Choice(choices=("person", "household")),
)
@click.option(
    "--download-path",
    prompt="path to download files to",
    default="../data/raw/",
    show_default=True,
    type=click.Path(exists=False, file_okay=False, writable=True),
)
@click.option(
    "--extract",
    prompt="extract zipped files?",
    default=True,
    show_default=True,
    type=click.BOOL,
)
@click.option(
    "--extract-path",
    prompt="path to extract contents to",
    default="../data/interim/",
    show_default=True,
    type=click.Path(exists=False, file_okay=False, writable=True),
)
def main(
    year, state, survey, person_or_household, download_path, extract, extract_path
):
    """Console script for acs_download."""
    pypums.get_data(
        year=year,
        state=state,
        survey=survey,
        person_or_household=person_or_household,
        download_path=download_path,
        extract=extract,
        extract_path=extract_path,
    )

    return print("Done!")


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



# -*- coding: utf-8 -*-

"""Top-level package for pypums."""

__author__ = """Sergio Sánchez Zavala"""
__email__ = "sergio@cimarron.io"
__version__ = '0.0.3'


from pypums.pypums import *
from pypums.surveys import ACS

if __name__ == "__main__":
    pypums.get_data()



# -*- coding: utf-8 -*-

"""Main module."""
# imports
from pathlib import Path
from typing import Union
from tqdm.auto import tqdm
from zipfile import ZipFile
import requests
import time
import us

import pypums
from pypums.download import download_acs_data
from pypums.url_builder import build_acs_url


def get_data(
    year: Union[int, str] = "2017",
    survey: Union[str, int] = "1-Year",
    person_or_household: str = "person",
    state: str = "California",
    download_path: str = "../data/raw/",
    extract: bool = True,
    extract_path: str = "../data/interim/",
):
    """
    Builds URL and downloads ACS 1-Year or 5-Year state PUMS estimates into a specified folder (defaults to ../data/raw/).
    """

    # builds URL
    URL = build_acs_url(
        year=year, survey=survey, person_or_household=person_or_household, state=state
    )

    # download data
    download_acs_data(
        url=URL, download_path=download_path, extract=extract, extract_path=extract_path
    )

    return None


## from realpython.com
def tree(directory):
    print(f"+ {directory}")
    for path in sorted(directory.rglob("*")):
        depth = len(path.relative_to(directory).parts)
        spacer = "    " * depth
        print(f"{spacer}+ {path.name}")


if __name__ == "__main__":
    get_data()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `pypums` package."""


import unittest
from click.testing import CliRunner

from pypums import pypums
from pypums import cli


class TestPypums(unittest.TestCase):
    """Tests for `pypums` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_000_something(self):
        """Test something."""

    def test_command_line_interface(self):
        """Test the CLI."""
        runner = CliRunner()
        result = runner.invoke(cli.main)
        assert result.exit_code == 0
        assert "Download complete!" in result.output
        help_result = runner.invoke(cli.main, ["--help"])
        assert help_result.exit_code == 0
        assert "Usage: main [OPTIONS]\n\n  Download" in help_result.output



from pathlib import Path
from tqdm.auto import tqdm
from zipfile import ZipFile
import requests
import time


def _check_data_folder(path: str = "../data/raw/", extract: bool = True):
    path = Path(path)

    if path.exists():
        pass
    else:
        Path("../data/").mkdir()
        Path("../data/raw/").mkdir()
        if extract:
            Path("../data/interim/").mkdir()

    return None


def download_acs_data(
    url: str,
    download_path: str = "../data/raw/",
    extract: bool = True,
    extract_path: str = "../data/interim/",
):
    """
    Downloads ACS 1-, 3-, or 5- estimates from a US Census Bureau's FTP-server URL.
    """

    # Checks download_path and extract_path exists
    _check_data_folder(path=download_path, extract=extract)

    # Downloads Data
    BASE_URL = "https://www2.census.gov/programs-surveys/acs/data/pums/"
    if not url[:55] == BASE_URL:
        raise ValueError(
            "Census FPT-server url's start with 'https://www2.census.gov/programs-surveys/acs/data/pums/'"
        )

    state = url.split("/")[-1].split(".")[0][-2:]

    chunk_size = 1024

    r = requests.get(url, stream=True)
    total_size = int(r.headers["content-length"])

    ### Checks
    download_path = Path(download_path)
    extract_path = Path(extract_path)

    if download_path.is_file():
        raise ValueError(
            "You provided a path to a file. You need to provide a path to a directory."
        )
    # if not download_path.is_dir():
    #     raise ValueError("You need to provide a path to a directory.")
    if not download_path.exists():
        download_path.mkdir()

    ### downloads data
    filename = url.split("/")[-1]

    with open(download_path / filename, "wb") as f:
        print(f"Downloading at {download_path / filename}.")
        for data in tqdm(
            iterable=r.iter_content(chunk_size=chunk_size),
            total=total_size / chunk_size,
            unit="KB",
        ):
            f.write(data)

    print("Download complete!")

    ## Extract file
    if extract:
        year = url.split("/")[7]
        extract_folder = f"ACS_{year}"

        final_extraction_folder = extract_path / extract_folder.upper() / state

        if extract_path.is_file():
            raise ValueError(
                "You provided a path to a file. You need to provide a path to a directory."
            )
        # if not extract_path.is_dir():
        #     raise ValueError("You need to provide a path to a directory.")
        if not extract_path.exists():
            extract_path.mkdir()

        # remove dir if it exists
        if final_extraction_folder.exists():
            for item in final_extraction_folder.glob("*"):
                item.unlink()
            final_extraction_folder.rmdir()

        # create dir
        if not Path(extract_path / extract_folder.upper()).exists():
            Path(extract_path / extract_folder.upper()).mkdir()
        final_extraction_folder.mkdir()

        # extracts data
        content_file = ZipFile(download_path / filename)

        ## for progress bar
        file_size = 0
        for file_info in content_file.infolist():
            file_size += int(file_info.file_size)

        extract_folder_size = sum(
            item.stat().st_size for item in final_extraction_folder.iterdir()
        )
        expected_final_size = extract_folder_size + file_size

        ## Start extraction:
        print(f"Extracting to {final_extraction_folder}")
        content_file.extractall(final_extraction_folder)
        while extract_folder_size < expected_final_size:
            extract_folder_size = sum(
                item.stat().st_size for item in final_extraction_folder.iterdir()
            )
            print(
                f"Extracting files to {final_extraction_folder}: {(extract_folder_size / file_size) :.2%}",
                end="\r",
            )
            time.sleep(0.5)
            break

        print(f"Files extracted successfully at {final_extraction_folder}")
